package org.acme;

import io.micrometer.core.instrument.MeterRegistry;
import io.micrometer.core.instrument.Tags;
import io.micrometer.core.instrument.Timer;
import io.quarkus.hibernate.orm.panache.PanacheEntityBase;
import org.acme.data.EnrolledSubject;
import org.acme.data.Subject;
import org.eclipse.microprofile.openapi.annotations.Operation;
import org.eclipse.microprofile.openapi.annotations.media.Content;
import org.eclipse.microprofile.openapi.annotations.media.Schema;
import org.eclipse.microprofile.openapi.annotations.parameters.Parameter;
import org.eclipse.microprofile.openapi.annotations.parameters.RequestBody;
import org.eclipse.microprofile.openapi.annotations.responses.APIResponse;

import javax.annotation.security.RolesAllowed;
import javax.enterprise.context.RequestScoped;
import javax.transaction.Transactional;
import javax.ws.rs.*;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import java.net.URI;
import java.util.List;

@Path("/enrolled-subjects")
@RequestScoped
@Produces(MediaType.APPLICATION_JSON)
@Consumes(MediaType.APPLICATION_JSON)
@RolesAllowed({"Admin", "Teacher"})
public class EnrolledSubjectsResource {

    private final MeterRegistry registry;

    EnrolledSubjectsResource(MeterRegistry registry) {
        this.registry = registry;
        registry.gaugeCollectionSize("enrolledSubjectsResource.EnrolledSubject.size", Tags.empty(), EnrolledSubject.listAll());
    }

    @GET
    @Operation(summary = "Get all EnrolledSubjects")
    @APIResponse(description = "Field of EnrolledSubjects",
            content = @Content(mediaType = "application/json",
            schema = @Schema(implementation = EnrolledSubject.class)))
    @APIResponse(responseCode = "200", description = "OK")
    @APIResponse(responseCode = "401", description = "Unauthorized")
    public List<EnrolledSubject> list() {
        registry.counter("enrolledSubjectsResource.listMethod.callCounter").increment();
        Timer timer = registry.timer("enrolledSubjectsResource.listMethod.timer");
        return timer.record(() -> EnrolledSubject.listAll());
    }

    @GET
    @Path("/{id}")
    @Operation(summary = "Get EnrolledSubject by id")
    @APIResponse(description = "EnrolledSubject",
            content = @Content(mediaType = "application/json",
                    schema = @Schema(implementation = EnrolledSubject.class)))
    @APIResponse(responseCode = "200", description = "OK")
    @APIResponse(responseCode = "401", description = "Unauthorized")
    @APIResponse(responseCode = "404", description = "EnrolledSubject was not found")
    public EnrolledSubject get(@Parameter(description = "The id of EnrolledSubject to be fetched", required = true) @PathParam("id") Long id) {
        registry.counter("enrolledSubjectsResource.getMethod.callCounter").increment();
        Timer timer = registry.timer("enrolledSubjectsResource.getMethod.timer");
        return timer.record(() -> (EnrolledSubject) EnrolledSubject
                .findByIdOptional(id)
                .orElseThrow(() -> {
                    throw new NotFoundException();
                }));
    }

    @POST
    @Transactional
    @RolesAllowed({"Admin", "Teacher", "Student"})
    @Operation(summary = "Creates EnrolledSubject")
    @APIResponse(responseCode = "201", description = "EnrolledSubject created")
    @APIResponse(responseCode = "400", description = "Bad request")
    @APIResponse(responseCode = "401", description = "Unauthorized")
    public Response create(@RequestBody(description = "Created EnrolledSubject object", required = true,
            content = @Content(schema = @Schema(implementation = EnrolledSubject.class))) EnrolledSubject enrolledSubject) {
        registry.counter("enrolledSubjectsResource.createMethod.callCounter").increment();
        Timer timer = registry.timer("enrolledSubjectsResource.createMethod.timer");
        return timer.record(() -> {
            Subject subject = Subject.findById(enrolledSubject.subjectId);
            if (subject == null) {
                return Response.status(Response.Status.BAD_REQUEST).build();
            }
            long alreadyEnrolledStudentsCount = EnrolledSubject.count("subjectId", subject.id);
            if (alreadyEnrolledStudentsCount < subject.studentLimit) {
                enrolledSubject.persist();
                return Response.created(URI.create("/enrolled-subjects/" + enrolledSubject.id)).build();
            }
            return Response.status(Response.Status.BAD_REQUEST).build();
        });
    }


    @DELETE
    @Path("/{id}")
    @Transactional
    @Operation(summary = "Delete EnrolledSubject by id")
    @APIResponse(description = "EnrolledSubject",
            content = @Content(mediaType = "application/json",
                    schema = @Schema(implementation = EnrolledSubject.class)))
    @APIResponse(responseCode = "204", description = "EnrolledSubject was deleted")
    @APIResponse(responseCode = "401", description = "Unauthorized")
    @APIResponse(responseCode = "404", description = "EnrolledSubject was not found")
    public void delete(@Parameter(description = "The id of EnrolledSubject to be deleted", required = true) @PathParam("id") Long id) {
        registry.counter("enrolledSubjectsResource.deleteMethod.callCounter").increment();
        Timer timer = registry.timer("enrolledSubjectsResource.deleteMethod.timer");
        timer.record(() -> {
            EnrolledSubject entity = EnrolledSubject.findById(id);
            if (entity == null) {
                throw new NotFoundException();
            }
            entity.delete();
        });
    }

    @DELETE
    @Transactional
    @Operation(summary = "Delete all EnrolledSubjects")
    @APIResponse(responseCode = "204", description = "All EnrolledSubject were deleted")
    @APIResponse(responseCode = "401", description = "Unauthorized")
    public void deleteAll() {
        registry.counter("enrolledSubjectsResource.deleteAllMethod.callCounter").increment();
        Timer timer = registry.timer("enrolledSubjectsResource.deleteAllMethod.timer");
        timer.record(() -> {
            EnrolledSubject.listAll().forEach(PanacheEntityBase::delete);
        });
    }

    @PUT
    @Path("/{id}")
    @Transactional
    @Operation(summary = "Update EnrolledSubject")
    @APIResponse(description = "EnrolledSubject",
            content = @Content(mediaType = "application/json",
                    schema = @Schema(implementation = EnrolledSubject.class)))
    @APIResponse(responseCode = "200", description = "EnrolledSubject was updated")
    @APIResponse(responseCode = "401", description = "Unauthorized")
    @APIResponse(responseCode = "404", description = "EnrolledSubject was not found")
    public EnrolledSubject update(@Parameter(description = "The id of EnrolledSubject to be updated", required = true) @PathParam("id")Long id,
                                  @RequestBody(description = "Updated EnrolledSubject object", required = true,
                                          content = @Content(schema = @Schema(implementation = EnrolledSubject.class))) EnrolledSubject enrolledSubject) {
        registry.counter("enrolledSubjectsResource.updateMethod.callCounter").increment();
        Timer timer = registry.timer("enrolledSubjectsResource.updateMethod.timer");
        return timer.record(() -> {
            EnrolledSubject enrolledSubjectById = EnrolledSubject.findById(id);
            if (enrolledSubject == null) {
                throw new NotFoundException();
            }
            if (enrolledSubject.mark != null) {
                enrolledSubjectById.mark = enrolledSubject.mark;
            }
            if (enrolledSubject.userId != null) {
                enrolledSubjectById.userId = enrolledSubject.userId;
            }
            if (enrolledSubject.subjectId != null) {
                enrolledSubjectById.subjectId = enrolledSubject.subjectId;
            }
            return enrolledSubjectById;
        });
    }
}
