package org.acme;

import io.micrometer.core.instrument.MeterRegistry;
import io.micrometer.core.instrument.Tags;
import io.micrometer.core.instrument.Timer;
import io.quarkus.hibernate.orm.panache.PanacheEntityBase;
import org.acme.data.Subject;
import org.eclipse.microprofile.openapi.annotations.Operation;
import org.eclipse.microprofile.openapi.annotations.media.Content;
import org.eclipse.microprofile.openapi.annotations.media.Schema;
import org.eclipse.microprofile.openapi.annotations.parameters.Parameter;
import org.eclipse.microprofile.openapi.annotations.parameters.RequestBody;
import org.eclipse.microprofile.openapi.annotations.responses.APIResponse;

import javax.annotation.security.PermitAll;
import javax.annotation.security.RolesAllowed;
import javax.enterprise.context.RequestScoped;
import javax.transaction.Transactional;
import javax.ws.rs.*;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import java.net.URI;
import java.util.List;

@Path("/subjects")
@RequestScoped
@Produces(MediaType.APPLICATION_JSON)
@Consumes(MediaType.APPLICATION_JSON)
@RolesAllowed({"Admin", "Teacher"})
public class SubjectResource {

    private final MeterRegistry registry;

    SubjectResource(MeterRegistry registry) {
        this.registry = registry;
        registry.gaugeCollectionSize("subjectResource.Subjects.size", Tags.empty(), Subject.listAll());
    }

    @GET
    @PermitAll
    @Operation(summary = "Get all subjects")
    @APIResponse(description = "Field of subjects",
            content = @Content(mediaType = "application/json",
            schema = @Schema(implementation = Subject.class)))
    @APIResponse(responseCode = "200", description = "OK")
    @APIResponse(responseCode = "401", description = "Unauthorized")
    public List<Subject> list() {
        registry.counter("subjectResource.listMethod.callCounter").increment();
        Timer timer = registry.timer("subjectResource.listMethod.timer");
        return timer.record(() -> Subject.listAll());
    }

    @GET
    @Path("/{id}")
    @PermitAll
    @Operation(summary = "Get subject by id")
    @APIResponse(description = "Subject",
            content = @Content(mediaType = "application/json",
            schema = @Schema(implementation = Subject.class)))
    @APIResponse(responseCode = "200", description = "OK")
    @APIResponse(responseCode = "401", description = "Unauthorized")
    @APIResponse(responseCode = "404", description = "Subject was not found")
    public Subject get(@Parameter(description = "The id of subject to be fetched", required = true) @PathParam("id") Long id) {
        registry.counter("subjectResource.getMethod.callCounter").increment();
        Timer timer = registry.timer("subjectResource.getMethod.timer");
        return timer.record(() -> (Subject) Subject
                .findByIdOptional(id)
                .orElseThrow(() -> {
                    throw new NotFoundException();
                }));
    }

    @GET
    @Path("/name/{name}")
    @PermitAll
    @Operation(summary = "Get subject by name")
    @APIResponse(description = "Subject",
            content = @Content(mediaType = "application/json",
                    schema = @Schema(implementation = Subject.class)))
    @APIResponse(responseCode = "200", description = "OK")
    @APIResponse(responseCode = "401", description = "Unauthorized")
    @APIResponse(responseCode = "404", description = "Subject was not found")
    public Subject getByName(@Parameter(description = "The name of subject to be fetched", required = true) @PathParam("name") String name) {
        registry.counter("subjectResource.getByNameMethod.callCounter").increment();
        Timer timer = registry.timer("subjectResource.getByNameMethod.timer");
        return timer.record(() -> (Subject) Subject.find("name", name).
                firstResultOptional()
                .orElseThrow(() -> {
                    throw new NotFoundException();
                }));
    }

    @POST
    @Transactional
    @Operation(summary = "Creates subject")
    @APIResponse(responseCode = "201", description = "Subject created")
    @APIResponse(responseCode = "400", description = "Bad request")
    @APIResponse(responseCode = "401", description = "Unauthorized")
    public Response create(@RequestBody(description = "Created subject object", required = true,
            content = @Content(schema = @Schema(implementation = Subject.class))) Subject subject) {
        registry.counter("subjectResource.createMethod.callCounter").increment();
        Timer timer = registry.timer("subjectResource.createMethod.timer");
        return timer.record(() -> {
            subject.persist();
            return Response.created(URI.create("/subjects/" + subject.id)).build();
        });
    }

    @DELETE
    @Path("/{id}")
    @Transactional
    @Operation(summary = "Delete subject by id")
    @APIResponse(description = "Subject",
            content = @Content(mediaType = "application/json",
                    schema = @Schema(implementation = Subject.class)))
    @APIResponse(responseCode = "204", description = "Subject was deleted")
    @APIResponse(responseCode = "401", description = "Unauthorized")
    @APIResponse(responseCode = "404", description = "Subject was not found")
    public void delete(@Parameter(description = "The id of subject to be deleted", required = true) @PathParam("id") Long id) {
        registry.counter("subjectResource.deleteMethod.callCounter").increment();
        Timer timer = registry.timer("subjectResource.deleteMethod.timer");
        timer.record(() -> {
            var entity = Subject.findByIdOptional(id).orElseThrow(() -> {
                throw new NotFoundException();
            });
            entity.delete();
        });
    }

    @DELETE
    @Transactional
    @Operation(summary = "Delete all subjects")
    @APIResponse(responseCode = "204", description = "All subjects were deleted")
    @APIResponse(responseCode = "401", description = "Unauthorized")
    public void deleteAll() {
        registry.counter("subjectResource.deleteAllMethod.callCounter").increment();
        Timer timer = registry.timer("subjectResource.deleteAllMethod.timer");
        timer.record(() -> {
            Subject.listAll().forEach(PanacheEntityBase::delete);
        });
    }

    @PUT
    @Path("/{id}")
    @Transactional
    @Operation(summary = "Update subject")
    @APIResponse(description = "Subject",
            content = @Content(mediaType = "application/json",
            schema = @Schema(implementation = Subject.class)))
    @APIResponse(responseCode = "200", description = "Subject was updated")
    @APIResponse(responseCode = "401", description = "Unauthorized")
    @APIResponse(responseCode = "404", description = "Subject was not found")
    public Subject update(@Parameter(description = "The id of subject to be updated", required = true) @PathParam("id") Long id,
                          @RequestBody(description = "Updated subject object", required = true,
                                  content = @Content(schema = @Schema(implementation = Subject.class))) Subject subject) {
        registry.counter("subjectResource.updateMethod.callCounter").increment();
        Timer timer = registry.timer("subjectResource.updateMethod.timer");
        return timer.record(() -> {
            var optional = Subject.findByIdOptional(id);
            if (optional.isEmpty()) {
                throw new NotFoundException();
            }
            var subjectById = (Subject) optional.get();
            if (subject.name != null) {
                subjectById.name = subject.name;
            }
            subjectById.teacher = subject.teacher;
            subjectById.studentLimit = subject.studentLimit;
            subjectById.persist();
            return subjectById;
        });
    }
}
