package org.acme;

import io.quarkus.test.common.http.TestHTTPEndpoint;
import io.quarkus.test.common.http.TestHTTPResource;
import io.quarkus.test.junit.QuarkusTest;
import io.quarkus.test.security.TestSecurity;
import io.restassured.http.ContentType;
import org.acme.data.EnrolledSubject;
import org.acme.data.Subject;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import javax.inject.Inject;
import java.io.IOException;
import java.net.URL;
import java.util.List;

import static io.restassured.RestAssured.given;
import static io.restassured.RestAssured.when;
import static org.hamcrest.CoreMatchers.containsString;
import static org.hamcrest.CoreMatchers.is;

@QuarkusTest
@TestHTTPEndpoint(EnrolledSubjectsResource.class)
@TestSecurity(authorizationEnabled = false)
public class EnrolledSubjectsResourceTest {

    @Inject
    SubjectResource subjectResource;

    @Inject
    EnrolledSubjectsResource enrolledSubjectsResource;

    @BeforeEach
    public void setup() {
        subjectResource.deleteAll();
        enrolledSubjectsResource.deleteAll();
    }

    @Test
    public void testGetEnrolledSubjectsWithNoEnrolledSubjectCreated() {
        when().get()
                .then()
                .statusCode(200)
                .body(is("[]"));
    }

    @Test
    public void testCreateEnrolledSubject() {
        Subject subject = new Subject();
        subject.name = "PA217";
        subject.teacher = 1;
        subject.studentLimit = 40;
        subjectResource.create(subject);

        given()
                .body(
                    "{\"userId\":1," +
                    "\"subjectId\":" + subject.id + "}")
                .contentType(ContentType.JSON)
                .when().post()
                .then().statusCode(201);

        List<EnrolledSubject> enrolledSubjects = enrolledSubjectsResource.list();

        Assertions.assertEquals(1, enrolledSubjects.size());
        Assertions.assertEquals(1, enrolledSubjects.get(0).userId);
        Assertions.assertEquals(subject.id, enrolledSubjects.get(0).subjectId);
    }

    @Test
    public void testUpdateEnrolledSubject() {
        Subject subject = new Subject();
        subject.name = "PA217";
        subject.teacher = 1;
        subject.studentLimit = 40;
        subjectResource.create(subject);

        EnrolledSubject enrolledSubject = new EnrolledSubject();
        enrolledSubject.subjectId = subject.id;
        enrolledSubject.userId = 1L;
        enrolledSubjectsResource.create(enrolledSubject);

        given().pathParam("id", enrolledSubject.id)
                .body(
                    "{\"userId\":1," +
                    "\"subjectId\":" + subject.id + "," +
                    "\"mark\":\"A\"}")
                .contentType(ContentType.JSON)
                .when().put("/{id}")
                .then().statusCode(200);

        List<EnrolledSubject> enrolledSubjects = enrolledSubjectsResource.list();

        Assertions.assertEquals(1, enrolledSubjects.size());
        Assertions.assertEquals(1, enrolledSubjects.get(0).userId);
        Assertions.assertEquals(subject.id, enrolledSubjects.get(0).subjectId);
        Assertions.assertEquals("A", enrolledSubjects.get(0).mark);
    }

    @Test
    public void testGetEnrolledSubjectById() {
        Subject subject = new Subject();
        subject.name = "PA217";
        subject.teacher = 1;
        subject.studentLimit = 40;
        subjectResource.create(subject);

        EnrolledSubject enrolledSubject = new EnrolledSubject();
        enrolledSubject.subjectId = subject.id;
        enrolledSubject.userId = 1L;
        enrolledSubjectsResource.create(enrolledSubject);

        given().pathParam("id", enrolledSubject.id)
                .when().get("/{id}")
                .then()
                .statusCode(200)
                .body(containsString(
            "\"userId\":1," +
                    "\"subjectId\":" + subject.id
                ));
    }

    @Test
    public void testDeleteEnrolledSubject() {
        Subject subject = new Subject();
        subject.name = "PA217";
        subject.teacher = 1;
        subject.studentLimit = 40;
        subjectResource.create(subject);

        EnrolledSubject enrolledSubject = new EnrolledSubject();
        enrolledSubject.subjectId = subject.id;
        enrolledSubject.userId = 1L;
        enrolledSubjectsResource.create(enrolledSubject);

        given().pathParam("id", enrolledSubject.id)
                .when().delete("/{id}")
                .then()
                .statusCode(204);

        List<EnrolledSubject> enrolledSubjects = enrolledSubjectsResource.list();
        Assertions.assertEquals(0, enrolledSubjects.size());
    }
}
