package org.acme;

import io.micrometer.core.instrument.MeterRegistry;
import io.micrometer.core.instrument.Tags;
import io.micrometer.core.instrument.Timer;
import io.quarkus.hibernate.orm.panache.PanacheEntityBase;
import org.acme.data.UserX;
import org.eclipse.microprofile.openapi.annotations.Operation;
import org.eclipse.microprofile.openapi.annotations.media.Content;
import org.eclipse.microprofile.openapi.annotations.media.Schema;
import org.eclipse.microprofile.openapi.annotations.parameters.Parameter;
import org.eclipse.microprofile.openapi.annotations.parameters.RequestBody;
import org.eclipse.microprofile.openapi.annotations.responses.APIResponse;

import javax.annotation.security.RolesAllowed;
import javax.enterprise.context.RequestScoped;
import javax.transaction.Transactional;
import javax.ws.rs.*;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import java.net.URI;
import java.util.List;

@Path("/users")
@RequestScoped
@Produces(MediaType.APPLICATION_JSON)
@Consumes(MediaType.APPLICATION_JSON)
@RolesAllowed("Admin")
public class UserResource {

    private final MeterRegistry registry;

    UserResource(MeterRegistry registry) {
        this.registry = registry;
        registry.gaugeCollectionSize("userResource.UserX.size", Tags.empty(), UserX.listAll());
    }

    @GET
    @Operation(summary = "Get all users")
    @APIResponse(description = "Field of users",
            content = @Content(mediaType = "application/json",
                    schema = @Schema(implementation = UserX.class)))
    @APIResponse(responseCode = "200", description = "OK")
    @APIResponse(responseCode = "401", description = "Unauthorized")
    public List<UserX> list() {
        registry.counter("userResource.listMethod.callCounter").increment();
        Timer timer = registry.timer("userResource.listMethod.timer");
        return timer.record(() -> UserX.listAll());
    }

    @GET
    @Path("/{id}")
    @RolesAllowed({"Admin", "Teacher"})
    @Operation(summary = "Get user by id")
    @APIResponse(description = "UserX",
            content = @Content(mediaType = "application/json",
                    schema = @Schema(implementation = UserX.class)))
    @APIResponse(responseCode = "200", description = "OK")
    @APIResponse(responseCode = "401", description = "Unauthorized")
    @APIResponse(responseCode = "404", description = "User was not found")
    public UserX get(@Parameter(description = "The id of USER to be fetched", required = true) @PathParam("id") Long id) {
        registry.counter("userResource.getMethod.callCounter").increment();
        Timer timer = registry.timer("userResource.getMethod.timer");
        return timer.record(() -> {
            var optional = UserX.findByIdOptional(id);
            if (optional.isEmpty()) throw new NotFoundException("User not found");
            return (UserX) optional.get();
        });
    }

    @GET
    @Path("/nickname/{nickname}")
    @RolesAllowed({"Admin", "Teacher"})
    @Operation(summary = "Get user by nickname")
    @APIResponse(description = "UserX",
            content = @Content(mediaType = "application/json",
                    schema = @Schema(implementation = UserX.class)))
    @APIResponse(responseCode = "200", description = "OK")
    @APIResponse(responseCode = "401", description = "Unauthorized")
    @APIResponse(responseCode = "404", description = "User was not found")
    public UserX getByNickname(@Parameter(description = "The nickname of user to be fetched", required = true) @PathParam("nickname")  String nickname) {
        registry.counter("userResource.getByNicknameMethod.callCounter").increment();
        Timer timer = registry.timer("userResource.getByNicknameMethod.timer");
        return timer.record(() -> {
            var optional = UserX.find("nickname", nickname).firstResultOptional();
            if (optional.isEmpty()) throw new NotFoundException("User not found");
            return (UserX) optional.get();
        });
    }

    @POST
    @Transactional
    @Operation(summary = "Creates user")
    @APIResponse(responseCode = "201", description = "User created")
    @APIResponse(responseCode = "400", description = "Bad request")
    @APIResponse(responseCode = "401", description = "Unauthorized")
    public Response create(@RequestBody(description = "Created user object", required = true,
            content = @Content(schema = @Schema(implementation = UserX.class))) UserX user) {
        registry.counter("userResource.createMethod.callCounter").increment();
        Timer timer = registry.timer("userResource.createMethod.timer");
        return timer.record(() -> {
            user.persist();
            return Response.created(URI.create("/users/" + user.id)).build();
        });
    }


    @DELETE
    @Path("/{id}")
    @Transactional
    @Operation(summary = "Delete user by id")
    @APIResponse(description = "UserX",
            content = @Content(mediaType = "application/json",
                    schema = @Schema(implementation = UserX.class)))
    @APIResponse(responseCode = "204", description = "User was deleted")
    @APIResponse(responseCode = "401", description = "Unauthorized")
    @APIResponse(responseCode = "404", description = "User was not found")
    public void delete(@Parameter(description = "The id of user to be deleted", required = true) @PathParam("id") Long id) {
        registry.counter("userResource.deleteMethod.callCounter").increment();
        Timer timer = registry.timer("userResource.deleteMethod.timer");
        timer.record(() -> {
            UserX entity = UserX.findById(id);
            if (entity == null) {
                throw new NotFoundException();
            }
            entity.delete();
        });
    }

    @DELETE
    @Transactional
    @Operation(summary = "Delete all users")
    @APIResponse(responseCode = "204", description = "All users were deleted")
    @APIResponse(responseCode = "401", description = "Unauthorized")
    public void deleteAll() {
        registry.counter("userResource.deleteAllMethod.callCounter").increment();
        Timer timer = registry.timer("userResource.deleteAllMethod.timer");
        timer.record(() -> {
            UserX.listAll().forEach(PanacheEntityBase::delete);
        });
    }

    @PUT
    @Path("/{id}")
    @Transactional
    @Operation(summary = "Update user")
    @APIResponse(description = "UserX",
            content = @Content(mediaType = "application/json",
                    schema = @Schema(implementation = UserX.class)))
    @APIResponse(responseCode = "200", description = "User was updated")
    @APIResponse(responseCode = "401", description = "Unauthorized")
    @APIResponse(responseCode = "404", description = "User was not found")
    public UserX update(@Parameter(description = "The id of user to be updated", required = true) @PathParam("id") Long id,
                        @RequestBody(description = "Updated UserX object", required = true,
                                content = @Content(schema = @Schema(implementation = UserX.class))) UserX user) {
        registry.counter("userResource.updateMethod.callCounter").increment();
        Timer timer = registry.timer("userResource.updateMethod.timer");
        return timer.record(() -> {
            UserX userById = UserX.findById(id);
            if (userById == null) {
                throw new NotFoundException();
            }
            if (user.nickname != null) {
                userById.nickname = user.nickname;
            }
            if (user.email != null) {
                userById.email = user.email;
            }
            if (user.roleName != null) {
                userById.roleName = user.roleName;
            }
            if (user.firstName != null) {
                userById.firstName = user.firstName;
            }
            if (user.lastName != null) {
                userById.lastName = user.lastName;
            }
            userById.persist();
            return userById;
        });
    }
}
